﻿Imports System.Runtime.CompilerServices
Imports System.IO
Imports System.IO.File
Imports System.Security
Imports System.Security.Cryptography

Module StringCryptoExtensions
    ' Encrypt a string.
    <Extension()> _
    Public Function Encrypt(ByVal plain_text As String, ByVal password As String) As Byte()
        Dim ascii_encoder As New System.Text.ASCIIEncoding()
        Dim plain_bytes As Byte() = ascii_encoder.GetBytes(plain_text)
        Return CryptBytes(password, plain_bytes, True)
    End Function

    ' Decrypt a string.
    <Extension()> _
    Public Function Decrypt(ByVal encrypted_bytes() As Byte, ByVal password As String) As String
        Dim decrypted_bytes() As Byte = CryptBytes(password, encrypted_bytes, False)
        Dim ascii_encoder As New System.Text.ASCIIEncoding()
        Return ascii_encoder.GetChars(decrypted_bytes)
    End Function

    ' Encrypt or decrypt a file, saving the results 
    ' in another file.
    Private Function CryptBytes(ByVal password As String, ByVal in_bytes() As Byte, ByVal encrypt As Boolean) As Byte()
        ' Make a triple DES service provider.
        Dim des_provider As New TripleDESCryptoServiceProvider

        ' Find a valid key size for this provider.
        Dim key_size_bits As Integer = 0
        For i As Integer = 1024 To 1 Step -1
            If des_provider.ValidKeySize(i) Then
                key_size_bits = i
                Exit For
            End If
        Next i
        Debug.Assert(key_size_bits > 0)

        ' Get the block size for this provider.
        Dim block_size_bits As Integer = des_provider.BlockSize

        ' Generate the key and initialization vector.
        Dim key As Byte() = Nothing
        Dim iv As Byte() = Nothing
        Dim salt As Byte() = {&H10, &H20, &H12, &H23, &H37, &HA4, &HC5, &HA6, &HF1, &HF0, &HEE, &H21, &H22, &H45}
        MakeKeyAndIV(password, salt, key_size_bits, block_size_bits, key, iv)

        ' Make the encryptor or decryptor.
        Dim crypto_transform As ICryptoTransform
        If encrypt Then
            crypto_transform = des_provider.CreateEncryptor(key, iv)
        Else
            crypto_transform = des_provider.CreateDecryptor(key, iv)
        End If

        ' Create the output stream.
        Dim out_stream As New MemoryStream()

        ' Attach a crypto stream to the output stream.
        Dim crypto_stream As New CryptoStream(out_stream, _
            crypto_transform, CryptoStreamMode.Write)

        ' Write the bytes into the CryptoStream.
        crypto_stream.Write(in_bytes, 0, in_bytes.Length)
        Try
            crypto_stream.FlushFinalBlock()
        Catch crypto_ex As CryptographicException
            ' Ignore this one. The password is bad.
        Catch ex As Exception
            ' Re-raise this one.
            Throw ex
        End Try

        ' Save the result.
        Dim result() As Byte = out_stream.ToArray()

        ' Close the stream.
        Try
            crypto_stream.Close()
        Catch crypto_ex As CryptographicException
            ' Ignore this one. The password is bad.
        Catch ex As Exception
            ' Re-raise this one.
            Throw ex
        End Try
        out_stream.Close()

        Return result
    End Function

    ' Use the password to generate key bytes.
    Private Sub MakeKeyAndIV(ByVal password As String, ByVal salt() As Byte, ByVal key_size_bits As Integer, ByVal block_size_bits As Integer, ByRef key As Byte(), ByRef iv As Byte())
        Dim derive_bytes As New Rfc2898DeriveBytes( _
            password, salt, 1234)

        key = derive_bytes.GetBytes(key_size_bits \ 8)
        iv = derive_bytes.GetBytes(block_size_bits \ 8)
    End Sub
End Module
