using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Reflection;

namespace CReflectionExploreOtherAssembly
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Look for the ReflectionTarget program.
        private void Form1_Load(object sender, EventArgs e)
        {
            DirectoryInfo dir_info = new DirectoryInfo(Application.StartupPath);
            dir_info = dir_info.Parent.Parent.Parent;
            txtAssembly.Text = dir_info.FullName + "\\CReflectionTarget\\bin\\Debug\\CReflectionTarget.exe";
        }

        // List the classes in the target.
        private void btnListClasses_Click(object sender, EventArgs e)
        {
            Assembly assem;
            try
            {
                assem = Assembly.LoadFile(txtAssembly.Text);
            }
            catch(FileNotFoundException)
            {
                MessageBox.Show("File not found. Make sure you have built the CReflectionTarget.exe project",
                    "File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            catch(Exception ex)
            {
                throw ex;
            }

            lstClasses.Items.Clear();
            ClearListBoxes();

            foreach (Type target_type in assem.GetTypes())
            {
                lstClasses.Items.Add(target_type);
            }
        }

        private void ClearListBoxes()
        {
            ListBox[] list_boxes = {lstConstructors, lstEvents, lstFields, lstInterfaces};

            foreach (ListBox lst in list_boxes)
            {
                lst.Items.Clear();
            }
        }

        // Display the methods available in this class.
        private void lstClasses_SelectedIndexChanged(object sender, EventArgs e)
        {
            ClearListBoxes();

            Type target_type = (Type)lstClasses.SelectedItem;
            if (target_type == null) return;

            foreach (ConstructorInfo ci in target_type.GetConstructors())
                lstConstructors.Items.Add(DescribeConstructor(ci));

            foreach (MethodInfo mi in target_type.GetMethods())
                lstMethods.Items.Add(new NamedObject(DescribeMethod(mi), mi));

            foreach (PropertyInfo pi in target_type.GetProperties())
            {
                lstProperties.Items.Add(new NamedObject(DescribeProperty(pi), pi));
                if (pi.GetGetMethod() != null)
                    lstProperties.Items.Add("    " + DescribeMethod(pi.GetGetMethod()));
                if (pi.GetSetMethod() != null)
                    lstProperties.Items.Add("    " + DescribeMethod(pi.GetSetMethod()));
            }

            foreach (FieldInfo fi in target_type.GetFields())
                lstFields.Items.Add((Object)new NamedObject(DescribeField(fi), fi));
            
            foreach (EventInfo ei in target_type.GetEvents())
                lstEvents.Items.Add(DescribeEvent(ei));

            foreach (Type itype in target_type.GetInterfaces())
                lstInterfaces.Items.Add(DescribeInterface(itype));
        }

        #region "Description Routines"

        // Return the constructor's description.
        private String DescribeConstructor(ConstructorInfo ci)
        {
            String result = "";
            result += DescribeAccessibility(ci);

            result += "Sub New" +
                DescribeParameterList(ci.GetParameters());

            return result;
        }

        // Return a method's description.
        private String DescribeMethod(MethodInfo mi)
        {
            String result = "";
            result += DescribeAccessibility(mi);

            if (mi.ReturnType.Name == "Void")
            {
                result += "Sub ";
            } else {
                result += "";
            }

            result += mi.Name +
                DescribeParameterList(mi.GetParameters());

            return result;
        }

        // Describe a property.
        private String DescribeProperty(PropertyInfo pi)
        {
            String result = "";

            if (!pi.CanRead) result += "WriteOnly ";
            if (!pi.CanWrite) result += "ReadOnly ";

            result += pi.Name + " As " + pi.PropertyType.ToString();

            return result;
        }

        // Describe a field.
        private String DescribeField(FieldInfo fi)
        {
            String result = "";
            result += DescribeAccessibility(
                false, false, false, fi.IsStatic,
                false, fi.IsPrivate, fi.IsPublic, fi.IsFamily,
                fi.IsFamilyAndAssembly, fi.IsFamilyOrAssembly, fi.IsAssembly);
            result += fi.Name + " As ";
            result += fi.FieldType.Name;

            return result;
        }

        // Describe an event.
        private String DescribeEvent(EventInfo ei)
        {
            return ei.Name;
        }

        // Describe an interface's type.
        private String DescribeInterface(Type itype)
        {
            return itype.Name;
        }

        #endregion

        #region "Helper Routines"
        // A class to hold a name and object.
        private class NamedObject
        {
            public String Name;
            public Object Data;
            public NamedObject(String new_name, Object new_data)
            {
                Name = new_name;
                Data = new_data;
            }
            public override String ToString()
            {
                return Name;
            }
        }

        // Return an object's accessibility.
        // Note: These combinations have not all been tested.
        private String DescribeAccessibility(MethodBase mb)
        {
            return DescribeAccessibility(
                mb.IsFinal, mb.IsAbstract, mb.IsHideBySig, mb.IsStatic,
                mb.IsVirtual, mb.IsPrivate, mb.IsPublic, mb.IsFamily,
                mb.IsFamilyAndAssembly, mb.IsFamilyOrAssembly, mb.IsAssembly);
        }

        // Return an object's accessibility.
        // Note: These combinations have not all been tested.
        private String DescribeAccessibility(bool is_final, bool is_abstract, bool is_hide_by_sig, bool is_static, bool is_virtual, bool is_private, bool is_public, bool is_family, bool is_family_and_assembly, bool is_family_or_assembly, bool is_assembly)
        {
            String result = "";

            if (is_private)
                result += "Private ";
            else if (is_public)
                result += "Public ";
            else if (is_family)
                result += "Protected ";
            else if (is_family_and_assembly)
                result += "Protected Friend ";
            else if (is_family_or_assembly)
                result += "ProtectedOrFriend "; // Should not happen.
            else if (is_assembly)
                result += "Friend ";

            if (is_final)
                result += "NotOverridable ";
            else if (is_abstract)
                result += "MustOverride ";
            else if (is_hide_by_sig)
                result += "Shadows ";
            else if (is_static)
                result += "Shared ";
            else if (is_virtual)
                result += "Overrides ";

            return result;
        }

        // Return a string describing a parameter.
        // Ends with ", ".
        private String DescribeParameter(ParameterInfo pi)
        {
            String result = "";

            if (pi.IsOptional) result += "Optional ";

            if (pi.IsOut)
                result += "ByRef ";
            else
                result += "ByVal ";

            result += pi.Name + " As " + pi.ParameterType.ToString();

            return result + ", ";
        }

        // Return a parameter list surrounded by parentheses.
        private String DescribeParameterList(ParameterInfo[] parameters)
        {
            String result = "";

            foreach (ParameterInfo pi in parameters)
                result += DescribeParameter(pi);

            // Remove the final ", ".
            if (result.Length > 0) result = result.Substring(0, result.Length - 2);

            return "(" + result + ")";
        }

        #endregion

        // Invoke a constructor.
        private void btnInvokeNew_Click(object sender, EventArgs e)
        {
            // Get the class.
            Type target_type = (Type)lstClasses.SelectedItem;

            // Get the parameters. (We assume these are strings.)
            String[] parameters = null;
            String txt = txtNewParameters.Text.Trim();
            if (txt.Length > 0) parameters = txt.Split(',');

            // Invoke the constructor.
            Object instance = target_type.InvokeMember(
                "", BindingFlags.CreateInstance, null, null, parameters);

            MessageBox.Show(instance.ToString(), "Result",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // Invoke a method.
        private void btnInvoke_Click(object sender, EventArgs e)
        {
            // Get the class.
            Type target_type = (Type)lstClasses.SelectedItem;

            // Create an instance of the class.
            Object instance = target_type.InvokeMember(
                "", BindingFlags.CreateInstance, null, null, null);

            // Get the MethodInfo from the ListBox.
            NamedObject named_object = (NamedObject)lstMethods.SelectedItem;
            MethodInfo method_info = (MethodInfo)named_object.Data;

            // Get the parameters. (We assume these are strings.)
            String[] parameters = null;
            String txt = txtParameters.Text.Trim();
            if (txt.Length > 0) parameters = txt.Split(',');

            // Invoke the method.
            Object result = method_info.Invoke(instance, parameters);
            if (result == null)
                MessageBox.Show("Done", "Done", MessageBoxButtons.OK, MessageBoxIcon.Information);
            else
                MessageBox.Show(result.ToString(), "Result", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // Create an object and set this property.
        private void btnSetProperty_Click(object sender, EventArgs e)
        {
            // Get the class.
            Type target_type = (Type)lstClasses.SelectedItem;

            // Create an instance of the class.
            Object instance = target_type.InvokeMember(
                "", BindingFlags.CreateInstance, null, null, null);

            // Get the PropertyInfo from the ListBox.
            NamedObject named_object = (NamedObject)lstProperties.SelectedItem;
            PropertyInfo property_info = (PropertyInfo)named_object.Data;

            // Get the value. (We assume it//s a string.)
            String value = txtPropertyValue.Text;

            // Set the value.
            property_info.SetValue(instance, value, null);

            // Display the result.
            MessageBox.Show(instance.ToString(), "Result", 
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        // Create an object and set a field value.
        private void btnSetField_Click(object sender, EventArgs e)
        {
            // Get the class.
            Type target_type = (Type)lstClasses.SelectedItem;

            // Create an instance of the class.
            Object instance = target_type.InvokeMember(
                "", BindingFlags.CreateInstance, null, null, null);

            // Get the PropertyInfo from the ListBox.
            NamedObject named_object = (NamedObject)lstFields.SelectedItem;
            FieldInfo field_info = (FieldInfo)named_object.Data;

            // Get the value. (We assume it//s a string.)
            String value = txtFieldValue.Text;

            // Set the value.
            field_info.SetValue(instance, value);

            // Display the result.
            MessageBox.Show(instance.ToString(), "Result",
                MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void lstConstructors_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnInvokeNew.Enabled = lstConstructors.SelectedItem != null;
        }
        private void lstMethods_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnInvoke.Enabled = lstMethods.SelectedItem != null;
        }
        private void lstProperties_SelectedIndexChanged(object sender, EventArgs e)
        {
            Object obj = lstProperties.SelectedItem;
            btnSetProperty.Enabled = (obj != null) && (obj is NamedObject);
        }
        private void lstFields_SelectedIndexChanged(object sender, EventArgs e)
        {
            btnSetField.Enabled = lstFields.SelectedItem != null;
        }
    }
}