using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Reflection;
using System.Resources;
using System.Collections;
using System.Media;

namespace CReflectionGetResources
{
    public partial class Form1 : Form
    {
        private Assembly m_TargetAssembly;

        public Form1()
        {
            InitializeComponent();
        }

        // Look for the ReflectionTarget program.
        private void Form1_Load(object sender, EventArgs e)
        {
            DirectoryInfo dir_info = new DirectoryInfo(Application.StartupPath);
            dir_info = dir_info.Parent.Parent.Parent;
            txtFile.Text = dir_info.FullName + "\\CReflectionTarget\\bin\\Debug\\CReflectionTarget.exe";
        }

        // List the target assembly's resources.
        private void btnList_Click(object sender, EventArgs e)
        {
            // Get the target assembly.
            try
            {
                m_TargetAssembly = Assembly.LoadFile(txtFile.Text);
            }
            catch (FileNotFoundException)
            {
                MessageBox.Show("File not found. Make sure you have built the CReflectionTarget.exe project",
                    "File Not Found", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            catch (Exception ex)
            {
                throw ex;
            }

            // List the target's manifest resource names.
            lstResourceFiles.Items.Clear();
            foreach (String str in m_TargetAssembly.GetManifestResourceNames())
                lstResourceFiles.Items.Add(str);
        }

        // List this file's resources.
        private void lstResourceFiles_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lstResourceFiles.SelectedIndex < 0) return;

            lstResources.Items.Clear();

            ResourceReader resource_reader = new ResourceReader(
                m_TargetAssembly.GetManifestResourceStream(lstResourceFiles.Text));

            foreach (DictionaryEntry dict_entry in resource_reader)
            {
                lstResources.Items.Add(new NamedObject(
                    dict_entry.Key,
                    dict_entry.Value));
            }

            resource_reader.Close();
        }

        // Display the selected resource.
        private void lstResources_SelectedIndexChanged(object sender, EventArgs e)
        {
            lblString.Text = "";
            picImage.Image = null;
            this.Cursor = Cursors.WaitCursor;
            Refresh();

            NamedObject resource_info = (NamedObject)lstResources.SelectedItem;
            switch (resource_info.Value.GetType().Name)
            {
                case "Bitmap":
                    picImage.Image = (Bitmap)resource_info.Value;
                    break;
                case "String":
                    lblString.Text = (String)resource_info.Value;
                    break;
                default:
                    // Try to play it as audio.
                    try
                    {
                        SoundPlayer player = new SoundPlayer();
                        UnmanagedMemoryStream mem_stream = (UnmanagedMemoryStream)resource_info.Value;
                        player.Stream = mem_stream;
                        player.PlaySync();
                        mem_stream.Seek(0, SeekOrigin.Begin);
                    }
                    catch(Exception)
                    {
                        MessageBox.Show(resource_info.Name + " has an unkown resource type", "Unkown Resource Type", MessageBoxButtons.OK);
                    }
                    break;
            }

            this.Cursor = Cursors.Default;
        }

        private class NamedObject
        {
            public Object Name;
            public Object Value;
            public NamedObject(Object new_key, Object new_value)
            {
                Name = new_key;
                Value = new_value;
            }
            public override String ToString()
            {
                return Name.ToString() + " (" + Value.GetType().ToString() + ")";
            }
        }
    }
}