Imports System.IO
Imports System.Reflection

Public Class Form1
    ' List our classes.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim assem As Assembly = Assembly.GetExecutingAssembly()

        lstClasses.Items.Clear()
        ClearListBoxes()

        For Each target_type As Type In assem.GetTypes
            lstClasses.Items.Add(target_type)
        Next target_type
    End Sub

    Private Sub ClearListBoxes()
        Dim list_boxes() As ListBox = _
            {lstConstructors, lstEvents, lstFields, lstInterfaces}

        For Each lst As ListBox In list_boxes
            lst.Items.Clear()
        Next lst
    End Sub

    ' Display the methods available in this class.
    Private Sub lstClasses_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstClasses.SelectedIndexChanged
        ClearListBoxes()

        Dim target_type As Type = DirectCast(lstClasses.SelectedItem, Type)
        If target_type Is Nothing Then Exit Sub

        For Each ci As ConstructorInfo In target_type.GetConstructors()
            lstConstructors.Items.Add(DescribeConstructor(ci))
        Next ci

        For Each mi As MethodInfo In target_type.GetMethods()
            lstMethods.Items.Add(New NamedObject(DescribeMethod(mi), mi))
        Next mi

        For Each pi As PropertyInfo In target_type.GetProperties()
            lstProperties.Items.Add(New NamedObject(DescribeProperty(pi), pi))
            If pi.GetGetMethod() IsNot Nothing Then
                lstProperties.Items.Add("    " & DescribeMethod(pi.GetGetMethod()))
            End If
            If pi.GetSetMethod() IsNot Nothing Then
                lstProperties.Items.Add("    " & DescribeMethod(pi.GetSetMethod()))
            End If
        Next pi

        For Each fi As FieldInfo In target_type.GetFields()
            lstFields.Items.Add(New NamedObject(DescribeField(fi), fi))
        Next fi

        For Each ei As EventInfo In target_type.GetEvents()
            lstEvents.Items.Add(DescribeEvent(ei))
        Next ei

        For Each itype As Type In target_type.GetInterfaces()
            lstInterfaces.Items.Add(DescribeInterface(itype))
        Next itype
    End Sub

    ' Return the constructor's description.
    Private Function DescribeConstructor(ByVal ci As ConstructorInfo) As String
        Dim result As String = ""
        result &= DescribeAccessibility(ci)

        result &= "Sub New" & _
            DescribeParameterList(ci.GetParameters())

        Return result
    End Function

    ' Return a method's description.
    Private Function DescribeMethod(ByVal mi As MethodInfo) As String
        Dim result As String = ""
        result &= DescribeAccessibility(mi)

        If mi.ReturnType.Name = "Void" Then
            result &= "Sub "
        Else
            result &= "Function "
        End If

        result &= mi.Name & _
            DescribeParameterList(mi.GetParameters())

        Return result
    End Function

    ' Describe a property.
    Private Function DescribeProperty(ByVal pi As PropertyInfo) As String
        Dim result As String = ""

        If Not pi.CanRead Then
            result &= "WriteOnly "
        End If
        If Not pi.CanWrite Then
            result &= "ReadOnly "
        End If

        result &= pi.Name & " As " & pi.PropertyType.ToString

        Return result
    End Function

    ' Describe a field.
    Private Function DescribeField(ByVal fi As FieldInfo) As String
        Dim result As String = ""
        result &= DescribeAccessibility( _
            False, False, False, fi.IsStatic, _
            False, fi.IsPrivate, fi.IsPublic, fi.IsFamily, _
            fi.IsFamilyAndAssembly, fi.IsFamilyOrAssembly, fi.IsAssembly)
        result &= fi.Name & " As "
        result &= fi.FieldType.Name

        Return result
    End Function

    ' Describe an event.
    Private Function DescribeEvent(ByVal ei As EventInfo) As String
        Return ei.Name
    End Function

    ' Describe an interface's type.
    Private Function DescribeInterface(ByVal itype As Type) As String
        Return itype.Name
    End Function

#Region "Helper Routines"
    ' A class to hold a name and object.
    Private Class NamedObject
        Public Name As String
        Public Data As Object
        Public Sub New(ByVal new_name As String, ByVal new_data As Object)
            Name = new_name
            Data = new_data
        End Sub
        Public Overrides Function ToString() As String
            Return Name
        End Function
    End Class

    ' Return an object's accessibility.
    ' Note: These combinations have not all been tested.
    Private Function DescribeAccessibility(ByVal mb As MethodBase) As String
        Return DescribeAccessibility( _
            mb.IsFinal, mb.IsAbstract, mb.IsHideBySig, mb.IsStatic, _
            mb.IsVirtual, mb.IsPrivate, mb.IsPublic, mb.IsFamily, _
            mb.IsFamilyAndAssembly, mb.IsFamilyOrAssembly, mb.IsAssembly)
    End Function

    ' Return an object's accessibility.
    ' Note: These combinations have not all been tested.
    Private Function DescribeAccessibility(ByVal is_final As Boolean, ByVal is_abstract As Boolean, ByVal is_hide_by_sig As Boolean, ByVal is_static As Boolean, ByVal is_virtual As Boolean, ByVal is_private As Boolean, ByVal is_public As Boolean, ByVal is_family As Boolean, ByVal is_family_and_assembly As Boolean, ByVal is_family_or_assembly As Boolean, ByVal is_assembly As Boolean) As String
        Dim result As String = ""

        If is_private Then
            result &= "Private "
        ElseIf is_public Then
            result &= "Public "
        ElseIf is_family Then
            result &= "Protected "
        ElseIf is_family_and_assembly Then
            result &= "Protected Friend "
        ElseIf is_family_or_assembly Then
            result &= "ProtectedOrFriend " ' Should not happen.
        ElseIf is_assembly Then
            result &= "Friend "
        End If

        If is_final Then
            result &= "NotOverridable "
        ElseIf is_abstract Then
            result &= "MustOverride "
        ElseIf is_hide_by_sig Then
            result &= "Shadows "
        ElseIf is_static Then
            result &= "Shared "
        ElseIf is_virtual Then
            result &= "Overrides "
        End If

        Return result
    End Function

    ' Return a string describing a parameter.
    ' Ends with ", ".
    Private Function DescribeParameter(ByVal pi As ParameterInfo) As String
        Dim result As String = ""

        If pi.IsOptional Then
            result &= "Optional "
        End If

        If pi.IsOut Then
            result &= "ByRef "
        Else
            result &= "ByVal "
        End If

        result &= pi.Name & " As " & pi.ParameterType.ToString

        Return result & ", "
    End Function

    ' Return a parameter list surrounded by parentheses.
    Private Function DescribeParameterList(ByVal params() As ParameterInfo) As String
        Dim result As String = ""

        For Each pi As ParameterInfo In params
            result &= DescribeParameter(pi)
        Next pi

        ' Remove the final ", ".
        If result.Length > 0 Then
            result = result.Substring(0, result.Length - 2)
        End If

        Return "(" & result & ")"
    End Function
#End Region ' Helper Routines

    ' Invoke a constructor.
    Private Sub btnInvokeNew_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInvokeNew.Click
        ' Get the class.
        Dim target_type As Type = lstClasses.SelectedItem

        ' Get the parameters. (We assume these are strings.)
        Dim params() As String = Nothing
        Dim txt As String = txtNewParameters.Text.Trim
        If txt.Length > 0 Then
            params = txt.Split(",")
        End If

        ' Invoke the constructor.
        Dim instance As Object = target_type.InvokeMember( _
            "", BindingFlags.CreateInstance, Nothing, Nothing, params)
        MessageBox.Show(instance.ToString, "Result", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    ' Invoke a method.
    Private Sub btnInvoke_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnInvoke.Click
        ' Get the class.
        Dim target_type As Type = lstClasses.SelectedItem

        ' Create an instance of the class.
        Dim instance As Object = target_type.InvokeMember( _
            "", BindingFlags.CreateInstance, Nothing, Nothing, Nothing)

        ' Get the MethodInfo from the ListBox.
        Dim named_object As NamedObject = DirectCast(lstMethods.SelectedItem, NamedObject)
        Dim method_info As MethodInfo = DirectCast(named_object.Data, MethodInfo)

        ' Get the parameters. (We assume these are strings.)
        Dim params() As String = Nothing
        Dim txt As String = txtParameters.Text.Trim
        If txt.Length > 0 Then
            params = txt.Split(",")
        End If

        ' Invoke the method.
        Dim result As Object = method_info.Invoke(instance, params)
        If result Is Nothing Then
            MessageBox.Show("Done", "Done", MessageBoxButtons.OK, MessageBoxIcon.Information)
        Else
            MessageBox.Show(result.ToString, "Result", MessageBoxButtons.OK, MessageBoxIcon.Information)
        End If
    End Sub

    ' Create an object and set this property.
    Private Sub btnSet_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetProperty.Click
        ' Get the class.
        Dim target_type As Type = lstClasses.SelectedItem

        ' Create an instance of the class.
        Dim instance As Object = target_type.InvokeMember( _
            "", BindingFlags.CreateInstance, Nothing, Nothing, Nothing)

        ' Get the PropertyInfo from the ListBox.
        Dim named_object As NamedObject = DirectCast(lstProperties.SelectedItem, NamedObject)
        Dim property_info As PropertyInfo = DirectCast(named_object.Data, PropertyInfo)

        ' Get the value. (We assume it's a string.)
        Dim value As String = txtPropertyValue.Text

        ' Set the value.
        property_info.SetValue(instance, value, Nothing)

        ' Display the result.
        MessageBox.Show(instance.ToString, "Result", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    ' Create an object and set a field value.
    Private Sub btnSetField_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSetField.Click
        ' Get the class.
        Dim target_type As Type = lstClasses.SelectedItem

        ' Create an instance of the class.
        Dim instance As Object = target_type.InvokeMember( _
            "", BindingFlags.CreateInstance, Nothing, Nothing, Nothing)

        ' Get the PropertyInfo from the ListBox.
        Dim named_object As NamedObject = DirectCast(lstFields.SelectedItem, NamedObject)
        Dim field_info As FieldInfo = DirectCast(named_object.Data, FieldInfo)

        ' Get the value. (We assume it's a string.)
        Dim value As String = txtFieldValue.Text

        ' Set the value.
        field_info.SetValue(instance, value)

        ' Display the result.
        MessageBox.Show(instance.ToString, "Result", MessageBoxButtons.OK, MessageBoxIcon.Information)
    End Sub

    Private Sub lstMethods_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstMethods.SelectedIndexChanged
        btnInvoke.Enabled = lstMethods.SelectedItem IsNot Nothing
    End Sub

    Private Sub lstProperties_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstProperties.SelectedIndexChanged
        Dim obj As Object = lstProperties.SelectedItem
        btnSetProperty.Enabled = (obj IsNot Nothing) AndAlso (TypeOf obj Is NamedObject)
    End Sub

    Private Sub lstFields_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstFields.SelectedIndexChanged
        btnSetField.Enabled = lstFields.SelectedItem IsNot Nothing
    End Sub

    Private Sub lstConstructors_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstConstructors.SelectedIndexChanged
        btnInvokeNew.Enabled = lstConstructors.SelectedItem IsNot Nothing
    End Sub
End Class
